# brightsign_source_release
#
# This class exists in order to automatically generate the files required for the BrightSign source release. It does three separate things:
#
# 1. The remote_sources task ensures that the filenames of source
#    tarballs are made available to the release-source script.
#
# 2. The husk_recipe task generates "husk" recipes for packages that
#    the source is not shipped for so that other recipes can depend on
#    them and vice-versa. This ensures that we're releasing the source
#    for and able to build all dependencies.
#
# 3. Creates a new source_release task that the above tasks and other
#    recipes that have more complex source release requirements can
#    hook into.
#
# The aim is for everything that is required to generate the source
# release for a given image to be built by the all_source_releases
# task.

# Licenses for stuff we need to include in our source release
REMOTE_SOURCES_INCLUDE_LICENSES = "\
	GPL \
	GPLv1+ \
	GPLv2 GPLv2+ GPL-v2 \
	GPLv3 GPLv3+ GPL-v3 \
	LGPL \
	LGPLv2 LGPLv2+ LGPL-v2 \
	LGPLv2.1 LGPLv2.1+ LGPL-v2.1 LGPL2.1 \
	Apache-2.0 \
	Artistic \
	Boost \
	BSD \
	NewBSD \
	bzip2 \
	Info-ZIP \
	jpeg \
	Libpng \
	MIT \
        openssl \
	PD \
	PSFv2 \
	Sleepycat \
	Zlib \
"

# Licenses for stuff we explicitly need to exclude from our source release
REMOTE_SOURCES_EXCLUDE_LICENSES = "\
	Proprietary \
	CLOSED \
"

do_source_release() {
    :
}
do_source_release[nostamp] = "1"

do_all_source_releases[recrdeptask] = "do_source_release"
addtask all_source_releases after do_source_release
do_all_source_releases() {
    :
}
do_all_source_releases[nostamp] = "1"

addtask source_release after do_fetch

python do_remote_sources() {
    import re

    # Let recipes override the sources list used for this. In
    # particular directfb needs to do this.
    src_uri = (bb.data.getVar('SOURCE_RELEASE_SRC_URI', d, True) or "").split()
    if len(src_uri) == 0:
        src_uri = (bb.data.getVar('SRC_URI', d, True) or "").split()
        if len(src_uri) == 0:
            return
    
    # Should use generic license stuff when we merge it. This is a bit
    # naughty because we end up combining both AND and OR and ignore
    # brackets.
    licenses = filter(bool, [ l.strip() for l in re.split('[&|()]', bb.data.getVar('LICENSE', d, True)) ])
    
    include_licenses = set(bb.data.getVar('REMOTE_SOURCES_INCLUDE_LICENSES', d, True).split())
    exclude_licenses = set(bb.data.getVar('REMOTE_SOURCES_EXCLUDE_LICENSES', d, True).split())
    include_pkg = False
    exclude_pkg = False    
    for lic in licenses:
        if lic in include_licenses:
            include_pkg = True
        elif lic in exclude_licenses:
            exclude_pkg = True
    
    if include_pkg and exclude_pkg:
        bb.fatal("Package ${PN} has conflicting licenses '${LICENSE}'")

    if exclude_pkg:
        bb.warn("${PN} license is excluded from remote_sources")
        return
    
    if not include_pkg:
        bb.fatal("Package ${PN} has licenses '${LICENSE}' which are unknown to remote_sources")
        return

    if src_uri == [ "unpublished" ]:
        bb.fatal("${PN} license means we should list remote_sources but the URI is %s" % src_uri)
        return
    
    localdata = bb.data.createCopy(d)
    bb.data.update_data(localdata)
    
    try:
        fetcher = bb.fetch2.Fetch(src_uri, localdata)
        bb.mkdirhier(bb.data.expand("${DEPLOY_DIR}/remote_sources/${MACHINE}", d))
        f = open(bb.data.expand("${DEPLOY_DIR}/remote_sources/${MACHINE}/${PN}", d), 'w')
        for l in fetcher.localpaths():
            if l.startswith(bb.data.getVar('DL_DIR', d, True)):
                f.write("%s\n" % l)
                f.close
    except bb.fetch2.BBFetchException, e:
        raise bb.build.FuncFailed(e)
}

# If we want to be sure that the information is up to date we have to
# generate it every time. :(
do_remote_sources[nostamp] = "1"
    
addtask remote_sources after do_fetch before do_source_release

do_clean_remote_sources() {
    rm -f "${DEPLOY_DIR}/remote-sources/${MACHINE}/${PN}"
}

addtask clean_remote_sources before do_clean

# Licenses for stuff we explicitly need to exclude from our source
# releases but need to include husk recipes for to keep the dependency
# chain going. If we're not shipping the source then we must need a
# husk recipe.
HUSK_RECIPES_INCLUDE_LICENSES = "${REMOTE_SOURCES_EXCLUDE_LICENSES}"

# This recipe generates husky recipes for all closed source recipes so
# that their dependencies can be built
python do_husk_recipe() {
    import re, string

    # Should use generic license stuff when we merge it. This is a bit
    # naughty because we end up combining both AND and OR and ignore
    # brackets.
    licenses = filter(bool, [ l.strip() for l in re.split('[&|()]', bb.data.getVar('LICENSE', d, True)) ])
    
    include_licenses = set(bb.data.getVar('HUSK_RECIPES_INCLUDE_LICENSES', d, True).split())
    include_pkg = False
    for lic in licenses:
        if lic in include_licenses:
            include_pkg = True
    
    if not include_pkg:
        bb.warn("${PN} license %s is excluded from husk_recipes" % string.join(licenses))
        return

    pn = bb.data.getVar('PN', d, True)
    depends = bb.data.getVar('DEPENDS', d, True).split()
    rdepends = bb.data.getVar('RDEPENDS', d, True).split()
    provides = bb.data.getVar('RDEPENDS', d, True).split()
    packages = bb.data.getVar('PACKAGES', d, True).split()

    # Some recipes have dependencies only in the source release
    extra_depends =  bb.data.getVar('SOURCE_RELEASE_EXTRA_DEPENDS', d, True)
    if extra_depends:
       depends = depends + extra_depends.split()

    # Let's work out what the dependencies are for this recipe
    bb.warn("RECIPE: %s" % pn)
    bb.warn("DEPENDS: %s" % depends)
    bb.warn("RDEPENDS: %s" % rdepends)
    bb.warn("PROVIDES: %s" % provides)
    bb.warn("PACKAGES: %s" % packages)

    try:
        os.makedirs(bb.data.expand("${DEPLOY_DIR}/oe-generated/meta-bcm7425/recipes/${PN}", d))
    except os.error:
        pass

    f = open(bb.data.expand("${DEPLOY_DIR}/oe-generated/meta-bcm7425/recipes/${PN}/${PN}.bb", d), 'w')
    f.write("# %s\n" % pn)
    f.write("#\n")
    f.write("# This is an automatically generated husk recipe. It exists only to allow\n")
    f.write("# things to depend on it and to depend on other things. It is used to ensure\n")
    f.write("# that the compilation of everything is tested by BrightSign when generating\n")
    f.write("# the source release.\n")
    f.write("\n")
    f.write("LICENSE = \"CLOSED\"\n")
    if len(depends) > 0:
        f.write("DEPENDS = \"%s\"\n" % string.join(depends))

    if len(rdepends) > 0:
        f.write("RDEPENDS = \"%s\"\n" % string.join(rdepends))

    if len(provides) > 0:
        f.write("PROVIDES = \"%s\"\n" % string.join(provides))

    if len(packages) > 0:
        f.write("PACKAGES = \"%s\"\n" % string.join(packages))

    # We need to generate the packages even if they are empty so they can be installed
    f.write("ALLOW_EMPTY = \"1\"")            

    f.close
}

# If we want to be sure that the information is up to date we have to
# generate it every time. :(
do_husk_recipe[nostamp] = "1"
    
addtask husk_recipe after do_fetch before do_source_release

do_clean_husk_recipe() {
    rm -f "${DEPLOY_DIR}/oe-generated/meta-bcm7425/recipes/${PN}/${PN}.bb"
}
do_clean_husk_recipe[nostamp] = "1"

addtask clean_husk_recipe before do_clean
